#include "combine.h"

// Implementations from class combine
namespace licencePlates {

void combinePlates::addFile (string fileName)
{   
    plateList list;
    try {
        plateList tmpList(fileName);
	list= tmpList;
    } catch (plateReadError e1) {
    	throw(e1);
    }
    addList(list);
}

double combinePlates::cartesianProd(vector <int> &whichSites) const
{
    double product= 1.0;
    //cout << "Cart prod" << endl;
    for (unsigned int i= 0; i < whichSites.size(); i++) {
    	product*= lists[whichSites[i]].size();
	//cout << lists[whichSites[i]].size() << endl;
    }
    //cout << product << endl;
    return product;
}

void combinePlates::makeMatches()
// make all matches for a set of lists
{
    int noLists= lists.size();
    combos= vector <matchList> (noLists * noLists);
    for (int i= 0; i < noLists; i++) {
	for (int j= i+1; j < noLists; j++) {
	    //cout << "Matching " << i << " and " << j << endl;
	    
	    matchList newList(lists[i], lists[j]);
	    combos[which_match(i,j)]= newList;
	    //cout << "Match " << i << " " << j << " " << 
	    //	combos[which_match(i,j)].getNoMatches() << endl;
	}
    }
}

int combinePlates::countMatchAll()
// Count the matches across all lists in the set
{
    vector <int> list (noLists());
    for (int i= 0; i < noLists(); i++)
    	list[i]= i;
    return countMatch (list);
}

int combinePlates::countMatch(vector <int> &sites) 
// Count the number of matches across the sites listed in the vector
{
    map<vector <int>, int>::iterator findPlate;
    findPlate= noSetMatches.find(sites);
    if (findPlate != noSetMatches.end()) {
    	//cout << "Matches for ";
	//for (unsigned int i= 0; i < sites.size(); i++) {
	//    cout << sites[i] << "  ";
	//}
	//cout << noSetMatches[sites] << endl;
    	return noSetMatches[sites];
    }
    int count= 0;
    int noSites= sites.size();
    if (noSites < 2) {
    	cerr << "Problem in countMatch" << endl;
	return 0;
    }
    vector <const matchList * > matches(noSites -1);
    for (int i= 0; i < noSites - 1; i++) {
    	matches[i]= &getComboElement(sites[i],sites[i+1]);
    } //matches[i] is the list of all matches between the i'th and the
    // i+1 th site
    
    vector <int> level (noSites - 1, 0);
    vector <int> part (noSites -1, 0);
    int depth= 0;  // Depth at which we are travesing the match tree
    
    //Insanely complex traverse of web of matches --- efficient 
    // More so than nicer recursive code would be
    for (int i= 0; i < matches[0]->getSize(); i++) {
    	if (matches[0]->getNoMatchesAt(i) == 0)
	    continue;
	level[0]= i;
	part[0]= 0;
    	depth= 0;
	while (1) {
	    
	    if (depth == noSites - 2) {
	    	count+= matches[depth]->getNoMatchesAt(level[depth]);
		level[depth]= 0;
		part[depth]= 0;
		depth--;
		if (depth < 0)
	    	    break;
		continue;
	    }
	    if (matches[depth]->getNoMatchesAt(level[depth])
	    	> part[depth]) {
		level[depth+1]= matches[depth]->getMatchAt
		    (level[depth],part[depth]);
		part[depth+1]= 0;
		part[depth]++;
		depth++;
		continue;
	    }
	    part[depth]= 0;
	    level[depth]= 0;
	    depth--;
	    if (depth < 0)
	    	break;
	}   
    }
    noSetMatches[sites]= count;
    return count;
}

matchList::matchList (const plateList &list1, const plateList &list2)
// Find all the matches between list one and list two
{
    noMatches= 0;
    //cout << "Matching" << endl;
    matches= vector<allMatches> (list1.size());
    for (int i= 0; i < list1.size(); i++) {
    	for (int j= 0; j < list2.size(); j++) {
	    if (list1.getElement(i) == list2.getElement(j)) {
	    	addMatch(i, j);
	    	//cout << "Plate " << list1.getElement(i)
		//    << " matches " << list2.getElement(j) << endl;
	    }
	}
    }
    //cout << list1.getName() << " + " << list2.getName () << " " <<
    //	noMatches << endl;
}






} // end of namespace
