#ifndef _COMBINE_H
#define _COMBINE_H
#include "readplates.h"
#include "match.h"
#include <vector>
#include <string>
#include <map>
// Class for manipulating combinations of platelists

// implementations are in combine.cpp

namespace licencePlates {

class allMatches {  // Get all matches for a particular plate in a list
    public:
	void addMatch(const int & j) {
	    matches.push_back(j);
	}
	const int noMatches() const {return matches.size();}
	const int getElement(int i) const {return matches[i];}
    private:
	vector <int> matches;
};

class matchList {	// Represents matches between two lists 
    public:
	matchList() : noMatches(0), matches(0) {}
		// Construct a list of matches from two plate lists
	matchList (const plateList &list1, 
		const plateList &list2);
	void addMatch(const int &i,const int &j)
	    {matches[i].addMatch(j); noMatches++;}
	const int getNoMatchesAt(const int &i)
	    const {return (matches[i].noMatches());}
	const int getMatchAt(const int &i, const int &j)
	    const {return (matches[i].getElement(j));}
	const allMatches &getElement(const int &i) const 
	    {return matches[i];}
	const int getNoMatches() const {return noMatches;}
	const int getSize() const {return matches.size();}
    private:
    	int noMatches;
	vector <allMatches> matches;
};


class lexLT 	// Lexicographical less than functor for matchClass
{
   public:
    bool operator()(const matchClass &lhs, const matchClass &rhs) const
    {return lhs.lexLT(rhs);};
};

class combinePlates {  // Represents all possible matches in a class
    public:
	void addList(const plateList &newList)
	    {lists.push_back(newList);}
	void addFile(string fileName);
	plateList &getElement(int i) {return lists[i];}
	const matchList &getComboElement(const int i, const int j) 
	    const {return combos[which_match(i,j)];}
	const matchList &getMatches(int i, int j) 
	    const { return combos[which_match(i,j)];}
    	// Number of Observations at site n
	int noObs(int n) {return lists[n].size();}
	int noLists() {return lists.size();}
	double cartesianProd(vector <int> &whichSites) const;
	void makeMatches(); // Make all matches for this list
    	int countMatchAll();  // Count matches across all lists
	int countMatch (vector <int> &sites);
	    	// Count matches across sites on this list of sites.
    private:
	vector <plateList> lists;
	vector <matchList> combos;
	map <matchClass, int, lexLT > noRelaxedMatches;
	map <vector <int>, int> noSetMatches;
	const int which_match (int i, int j)
	    const{return lists.size()*i+j;}
};




} // end of namespace licencePlates

#endif
