#include "evaluate.h"
#include <math.h>

using namespace licencePlates;

double evaluate(combinePlates &obslist)
// Evaluate matches on this list
{
    vector <polynomial *> plist(obslist.noLists());
    vector <int> whichSites(obslist.noLists());
    
    //cerr << "Generating equations" << endl; 
    for (int i= 0; i < obslist.noLists(); i++) {
    	plist[i]= new polynomial(i+1);
	
	whichSites[i]= i;
    }
    //cout << (*plist[obslist.noLists()-1]) << endl;
    //cerr << "Generating matches" << endl;
    double count= evaluateSites(plist, obslist, whichSites);
    
    for (int i= 0; i < obslist.noLists(); i++) {
    	delete(plist[i]);
    }
    return count;
}

double evaluateSites(vector <polynomial *> &plist, 
    combinePlates &obslist, vector <int> &whichSites)
    
{   
    double lhsCount= 1;
    double rhsCount= 0;
    int noSites= whichSites.size();
    
    if (noSites < 1) {
    	cerr << "Problem in evaluateSites" << endl;
	return 0;
    }
    if (noSites == 1) {
    	double noObs= obslist.noObs(whichSites[0]);
	return noObs;
    }
    
    for (int i= 0; i < plist[noSites-1]->length(); i++) {
    	const polyElement *pe;
	pe= plist[noSites-1]->getElement(i);
	//pe->putTo(cout);
	//cout << endl;
	double lhsAdd= pe->lhsEvaluate(plist, obslist, whichSites);
	lhsCount-= lhsAdd;
	//cout << "lhs count " << lhsAdd << endl;
	double rhsAdd= pe->rhsEvaluate(plist, obslist, whichSites);
	rhsCount+= rhsAdd;
	//cout << "rhs count " << rhsAdd << endl;
    }
    //cout << "rhs " << rhsCount << " lhs " << lhsCount << endl;
    //cout << "Sites: ";
   // for (int i= 0; i < noSites; i++) {
    	//cout << whichSites[i] << " ";
    //}
    //cout << rhsCount/lhsCount << endl;
    return rhsCount/lhsCount;
}

double matchProb (int n)
// Function to calculate p(n)
{
    if (n <= 1)
    	return 1;
    return pow(0.0001, (n-1));
}

