#ifndef MATCH_H
#define MATCH_H
#include <vector>
#include <iostream>
using namespace std;

// Class for types of match and transversal of all types

// Definitions for matchClass and matchTrans
// Implementations are in matchimpl.cpp
// Template based implementations are in this header (necessary
// as of gcc 2.95 and earlier)

class matchClass {      //  See papers on the subject for full description
                        //  Class represents a type of match
    public:
	matchClass (): n(0),  x(0),height(0) {} 
			// Default constructor - empty - no sites
		
	matchClass(int width)  :
			n(width),  x(width, 1), height(1) {}
			// Constructor of (1, 1, .... 1) True match for "n" sites
	matchClass(const matchClass &match, int add);  
                    // Add one onto existing class	 
        bool isValid(void); // Check if this is a valid match class
	bool lexLT (const matchClass &) const;  // Lexicographical less than.	
        const int getWidth() const {return n;}
        const int getHeight() const {return height;}
 	const int getElement(const int j) const {return x[j];}
    
    private:
        
        int n;                // Number of elements in this match
	vector <int> x;     // Parts of our match
        int height;           // Height of match (no of distinct elements)
  	int calcHeight(void);  // Calculate height of matching class 
	friend ostream& operator << (ostream &os, const matchClass &right);
        friend bool operator == (const matchClass &left, 
            const matchClass &right);
	friend bool operator != (const matchClass &left,
	    const matchClass &right) 
	    {return !(left==right);}
        friend bool operator > (const matchClass &left, 
            const matchClass &right);
        friend bool operator < (const matchClass &left, 
            const matchClass &right);
};

template <class T> class matchTrans {
	// Transversal of all possible matches for n sites
	public: 
	    matchTrans(int n); 
                        // Construct the n'th transversal
	    matchTrans (int n, int classes, vector <T> xnew):
		sites(n), noClasses(classes), x(xnew) {};
	    T &getElement(int i)		// Return i'th element
		{return (x[i]);}
	    static int Stirling (int n, int k);  // Stirling no S(n,k)
	    static int calcClasses (int n);	// No of classes in n'th transversal
	    int getSites() const {return sites;}
	    int getNoClasses() const {return noClasses;}
	    matchTrans nextMatch ();  // Return the next transveral up
	    int countHeight(int h);	// Return the number of transversal
								// Elements with height h
            void drawTrans(bool swapAxes= false,
			int xWid= 140, int yWid=180); // Draw a transversal
			// Including arrows - using psfig.  Only defined for
			// matchDraw type transversals.
		
	private:
	    int sites;   // Number of sites for this transversal
	    int noClasses; // Number of matching classes in transversal
	    vector <T> x; // Classes in transversal
        template <class C> friend ostream& operator << 
	    (ostream &os, matchTrans <C> right);
};


template <class T> matchTrans<T>::matchTrans(int n)
{
	if (n < 1) {
		noClasses= 0;
		sites= 0;
		x= vector<T> (0);
		return;
	}
	if (n == 1) {
		noClasses= 1;
		sites= 1;
		x= vector <T> (1,1); // Set up a vector with 1 match 
		return;		 // The (1) Class
	}
	matchTrans Mprev= matchTrans<T>(n-1);
	
	// This duplicates code in nextMatch - horrible (but seemingly
	// unavoidable in C++)
	
	sites= n;
	noClasses= calcClasses(n);
	x= vector<T> (noClasses);
	
	int k= 0;
	for (int i= 0; i < Mprev.getNoClasses(); i++) {
		T z= Mprev.getElement(i);
		for (int j= 1; j <= z.getHeight() + 1; j++) {
			x[k]= T(z, j);
			k++;
		}
	}
}

template <class T> matchTrans<T> matchTrans<T>::nextMatch ()
{
	int newnoSites= getSites()+1;
	int newnoClasses= calcClasses(newnoSites);
	vector <T> newx(newnoClasses,0);	
	int k= 0;
	for (int i= 0; i < getNoClasses(); i++) {
		T z= getElement(i);
		for (int j= 1; j <= z.getHeight() + 1; j++) {
			newx[k]= T(z, j);
			k++;
		}
	}
	return matchTrans<T>(newnoSites,newnoClasses, newx);
}

template <class T> int matchTrans<T>::calcClasses (int n)
// Count the number of classes in a particular type of match
// Currently done with Stirling nos - replace when I find a
// more efficient method
{
	int classCount= 0;
	for (int k= 1; k <= n; k++) {
		classCount+= Stirling(n, k);
	}
	return classCount;
	
}

template <class T>int matchTrans<T>::Stirling (int n, int k) 
{
	if (k <= 1 || k >= n)
		return 1;
	if (n <= 1)
		return 1;
	return (Stirling (n-1, k-1) + k * Stirling(n-1, k));
}

template <class T> ostream& operator << (ostream &os, matchTrans <T> right)
// Output for match class
{
    
    for (int i= 0; i < right.noClasses; i++) {
        cout << right.x[i] << endl; 
    }
	return os;
}

#endif

