#include <iostream>
#include "match.h"
#include "matchdraw.h"

bool matchClass::isValid(void)
// Checks if this meets criterion for matching classes
{
	int h= 1;
	if (n == 0)
		return true;
	if (x[0] != 1)
		return false;
	for (int i= 1; i < n; i++) {
		if (x[i] < 1 || x[i] > h+1)
			return false;
		if (x[i] > h)
			h++;
	}
	return true;
}

bool matchClass::lexLT (const matchClass &rhs) const
{
    int rWidth= rhs.getWidth();
    int lWidth= getWidth();

    for (int i= 0; i < rWidth; i++) {
    	if (i == lWidth)
	    return true;
	if (getElement(i) < rhs.getElement(i))
	    return true;
	if (getElement(i) > rhs.getElement(i))
	    return false;
    }
    return false;

}

matchClass::matchClass(const matchClass &match, int add) 
// Add one element on the end of a match class vector
{
	n= match.getWidth() + 1;
	x= vector <int> (n, 1);
	for (int i= 0; i < n-1; i++)
		x[i]= match.getElement(i);
	x[n-1]= add;
	height= calcHeight();			
}


int matchClass::calcHeight(void)
{
	int h= 1;
	for (int i= 0; i < n; i++) {
		if (x[i] > h) 
			h= x[i];
	}
	return h;
}
   
ostream& operator << (ostream &os, const matchClass &right)
// Output for match class
{
    cout << "(";

    for (int i= 0; i < right.n; i++) {
        cout << right.x[i];
        if ( i != right.n - 1) 
            cout << ",";
    }
    cout << ")";
	return os;
}

bool operator == (const matchClass &left, const matchClass &right)
{
    int w;
    if ((w= left.getWidth()) != right.getWidth()) {
    	//cout << left << "!=" << right << endl;
        return false;
    }
    for (int i= 0; i < w; i++) {
        if (left.getElement(i) != right.getElement(i)) {
            //cout << left << "!=" << right << endl;
	    return false;
    	}
    }
   // cout << left << "==" << right << endl;
    return true;
}

bool operator > (const matchClass &left, const matchClass &right)
{
    int w;
    if ( (w= left.getWidth()) != right.getWidth())
        return false;
    if (left == right)
    	return false;
    for (int i= 0; i < w; i++) {
        for (int j= 0; j < w; j++) {
            if (i == j)
                continue;
            if (left.getElement(i) == left.getElement(j) &&
                right.getElement(i) != right.getElement(j))
                    return false;
            
        }
    }
    return true;
}

bool operator < (const matchClass &left, const matchClass &right)
{

    return (right > left);
}

template <> void matchTrans<matchDraw>::drawTrans 
	(bool swap, int xWid, int yWid)
// Draw out the Transversal including arrows.
{
        double redval= 0;
        double blueval= 0;
        double greenval= 0;

	setAxisRotation(swap);
	cout << "\\documentclass{article}" << endl;
	cout << "\\usepackage{epsfig}" << endl;
	cout << "\\usepackage{pstricks}" << endl;
        cout << "\\usepackage{pst-node}" << endl;
	cout << "\\begin{document}" << endl;
	cout << "%Latex Figure created by matching program" << endl;
	cout << "\\begin{figure}" << endl;
	cout << "\\begin{center}" << endl;
	cout << "\\unitlength=1mm" << endl;
	cout << "\\psset{unit=1mm}" << endl;
        cout << "\\psset{linewidth=0.5pt}" << endl;
	if (swap) {
	    cout << "\\psset{swapaxes=true}" << endl;
            cout << "\\begin{picture}(" << yWid << "," << xWid <<")" << endl;
        } else {
            cout << "\\begin{picture}(" << xWid << "," << yWid <<")" << endl;
        }
	for (int i= sites; i > 0; i--) {
	    int no= countHeight(i);
	    int xMult= 1;
	    for (int j= 0; j < noClasses; j++) {
		if (x[j].getHeight() != i)
			continue;
		x[j].setXY( (xMult)*xWid/(no+1),(sites-i)*yWid/(sites-1));
		xMult++;
	    }
	}
	for (int i= 0; i < noClasses; i++) {
            cout << x[i] << endl;
        }	
        cout.precision(2);
        cout.setf(ios::fixed, ios::floatfield);
	for (int i= 0; i < noClasses; i++) {
            redval+= 1.0;
            if (redval > 1.0) {
                redval= 0.0;
                greenval+= 1.0;
                if (greenval > 1.0) {
                    greenval= 0.0;
                    blueval+= 1.0;
                    if (blueval > 1.0) {
                        blueval= 0.0;
                    }
                }   
            }
            if (blueval == 1.0 && redval == 1.0 && greenval == 1.0) {
                blueval= 0.0;
                redval= 0.0;
                greenval= 0.0;
            }
            //cerr << "RGB: " << redval << " " << greenval << " " << blueval
            //  << endl;
            for (int j= 0; j < noClasses; j++) {
                if (j == i)
                    continue;
                if (x[i].getHeight() == x[j].getHeight() + 1 && x[i] > x[j]) {
                    cout << "\\newrgbcolor{tmpcolor}{" <<
                         redval << " " << greenval << " " << 
                         blueval << "}"
                          << endl;
                    
                    
                    cout << "\\psset{linecolor=tmpcolor}" << endl;
                    cout << "\\ncdiag[arm= 5pt, angleA=";
                    if (swap == true)
                        cout << "90";
                    else 
                        cout <<"0";
 
                    cout << ", angleB=";
                    if (swap == true)
                        cout << "270";
                    else 
                        cout <<"180";
 

                    cout << "]{->}{N" << x[i].getId() 
                        << "}{N" << x[j].getId() << "}" << endl;
                }
                     
            }
        }
        cout << "\\psset{linecolor=black}" << endl;
	cout << "\\end{picture}" << endl;
	cout << "\\end{center}" << endl;
	cout << "\\end{figure}" << endl;
	cout << "\\end{document}" << endl;
}

template <class T>int matchTrans<T>::countHeight(int h)
{
	int no= 0;
	for (int i= 0; i < noClasses; i++) {
		if (x[i].getHeight() == h)
			no++;
	}
	return no;
}
