#include "poly.h"
#include "evaluate.h"
#include <iostream>

ostream & operator << (ostream &os, const polyElement &p) 
// printout operator for polyElement class;
{
    p.putTo(os);
    return os;
}


void polyElement::putTo (ostream &os) const
{
    os << "Illegal Call to Base Class element " << endl;
}

bool polyElement::polyAddTo(vector <polyElement *> elems, 
	    polyElement *match)
{
    for (vector<polyElement *>::iterator i= elems.begin();
    	 i != elems.end(); i++) {
	if (match->equals(*i)) {
	    (*i)->addFactor(match->getFactor());
	    return true;
	}
    }
    return false;
}

void matchTrue::putTo (ostream &os) const
{
    if (mult == -1) {
    	os << "-";
    } else if (mult != 1) {
	os << mult << ".";
    }
    if (prob != 1)
    	os << "p(" << prob << ") ";
    os << "X(M" << getNoSites() << "(T),C(S" << getNoSites() << "))";
}

bool matchTrue::equals (polyElement *mt)
{
    matchTrue *mtr= dynamic_cast <matchTrue *>(mt);
    if (mtr== NULL) {
    	return false;
    }
    if (getProb() != mtr->getProb()) 
    	return false; 
    return true;
}

double matchTrue::lhsEvaluate(vector <polynomial *> &plist, 
    	    licencePlates::combinePlates &obslist, 
    	    vector <int> &whichSites) const 
{
    
    return 0;
}
	
double matchTrue::rhsEvaluate(vector <polynomial *> &plist, 
    	    licencePlates::combinePlates &obslist, 
	    vector <int> &whichSites) const

{
    double noMatches= obslist.countMatch(whichSites);
    return noMatches*matchProb(getProb())*getFactor();
}

void exactMatch::putTo (ostream &os)  const
{
    if (mult == -1) {
    	os << "-";
    } else if (mult != 1) {
	os << mult << ".";
    }
    if (prob != 1)
    	os << "p(" << prob << ") ";    
    os << "X(" << static_cast <const matchClass> (mc) << 
    	",S" << getNoSites() << ")";
}

bool exactMatch::equals (polyElement *mt)
{
    exactMatch *em= dynamic_cast <exactMatch *>(mt);
    if (em == NULL) {
    	return false;
    }
    if (getProb() != em->getProb()) 
    	return false;
    if (getMatchClass() != em->getMatchClass())
    	return false;
    return true;
}

void matchByParts::putTo (ostream &os) const
{
    if (mult == -1) {
    	os << "-";
    } else if (mult != 1) {
	os << mult << ".";
    }
    if (prob != 1)
    	os << "p(" << prob << ") ";  
    os << "R(" << mc << ",S" << getNoSites() << ")";
    //os << "match class is " << mc;
}

bool matchByParts::equals (polyElement *mt)
{
    matchByParts *mbp= dynamic_cast <matchByParts *>(mt);
    if (mbp == NULL) {
    	return false;
    }
    if (getProb() != mbp->getProb()) 
    	return false; 
    if (getMatchClass() != mbp->getMatchClass()) 
    	return false;
    return true;
}

double matchByParts::lhsEvaluate(vector <polynomial *> &plist, 
    	    licencePlates::combinePlates &obslist, 
    	    vector <int> &whichSites) const 
{
    if (mc.getHeight() == 1) {	    // height 1 is true match
    	return matchProb(getProb())*getFactor();
    }
    return 0;
}

double matchByParts::rhsEvaluate(vector <polynomial *> &plist, 
    	    licencePlates::combinePlates &obslist, 
	    vector <int> &whichSites) const 
{
    //cout << "MBP height " << mc.getHeight() << " sites size " <<
    //	whichSites.size() << endl;
    if (mc.getHeight() == 1) {
    	return 0;
    }
    int noSites= whichSites.size();
    if (noSites == mc.getHeight() ) {
    	return matchProb(getProb())*getFactor()*
	    obslist.cartesianProd(whichSites);
    }
    double mbp= 1;
    //putTo(cout);
    //cout << endl;
    for (int i= 1; i <= mc.getHeight(); i++) {
    	vector <int> siteList;
	//cout << " at level " << i << " matches elements ";
	for (int j= 0; j < mc.getWidth(); j++) {
	    
	    if (mc.getElement(j) == i) {
	    	//cout << j << " ";
	    	siteList.push_back(whichSites[j]);
	    }
	}
	
	double mult= evaluateSites(plist, obslist, siteList);
	mbp*= mult;
	//cout << " and has " << mult << " estimated matches" << endl;
    }
    return matchProb(getProb())*getFactor()*mbp;
}

vector <polyElement *> exactMatch::getExpansion (matchTrans <matchClass> &Mn,
    vector <polyElement *>elems)
// exact match of a particular type expands into a relaxed match
// of a particular type
{
    vector <polyElement *> pv;
    matchByParts *mbp= new matchByParts(getNoSites(), getMatchClass(), 
    	getProb(), getFactor());
    pv.push_back(mbp);
    
    int nc=  Mn.getNoClasses();
    
    for (int i= 0; i < nc; i++) {
    	//cout << "n = " << i << " out of " << nc << endl;
    	matchClass &mClass = Mn.getElement(i);
	if (!(mClass < mc))
	    continue;	
	//cout << "Adding match of class " << mClass << endl;
	exactMatch *xmat= new exactMatch(getNoSites(), mClass, getProb(),
	     -getFactor());
	if (polyAddTo(elems, xmat)) {
	    delete(xmat);
	} else {
	    pv.push_back(xmat);
	}
	//cout << "Added match: ";
	//xmat->putTo(cout);
	//cout << endl;
    }
    return pv;
}

polynomial::polynomial (int n) :
    noSites(n), Mn(n)
//  Construct the polynomial for match transversal M_n
{
    matchTrue *mt= new matchTrue (n);
    elements.push_back(mt);
    int nc=  Mn.getNoClasses();
    exactMatch *me;
    matchClass * mc;
    for (int i= 0; i < nc; i++) {
    	 mc= &(Mn.getElement(i));
	 if (mc->getHeight() == 1)  // Don't add the "true match" class
	    continue;
	 me= new exactMatch(n, (*mc), mc->getHeight(), -1);
	 elements.push_back(me);
    }
    //putTo(cout);
    //cout << endl;
    expandPoly();
}

polynomial::~polynomial()
{
    int nel= length();
    for (int i= 0; i < nel; i++) {
    	//cout << "Deleted element " << i << endl;
    	delete(elements[i]);
    }
}

void polynomial::expandPoly()
// Expand terms of polynomial
{
    //cout << "EXPANDING ONCE" << endl;
    //putTo(cout);
    //cout << endl;
    vector <polyElement *>addvect;
    for (vector<polyElement *>::iterator i= elements.begin();
    	 i != elements.end(); i++) {
    	 if ((*i)->isExpansible()) {
	    //cout << "Expanding:  ";
	    //(*i)->putTo(cout);
	    //cout << endl;
	    addvect= (*i)->getExpansion(Mn, elements);  // Add expansion onto the end
	    delete(*i);     	// Delete the memory saved for the vector
	    elements.erase(i);	// And remove it from the vector
	    for (size_t j= 0; j < addvect.size(); j++) {
	    	elements.push_back(addvect[j]);
	    }
	    expandPoly();   	// Restart the expansion and leave this
	    return; 	    	// function
	}  
    }
    //cout << "FINISHED EXPANSION" << endl;
}

void polynomial::putTo (ostream &os) const

{
    os << "M" << getNoSites() << "(T,S" << getNoSites() << ") = ";
    for (int i= 0; i < length(); i++) {
    	const polyElement *pe= getElement(i);
	if (i != 0) {
	    if (pe->getFactor() > 0) 
	    	os << " +";
	    else 
	    	os << " ";
	}
	os << (*pe);
    }


}

ostream & operator << (ostream &os, const polynomial &p) 
//

{
    p.putTo(os);
    return os;
}
