#ifndef _POLY_H
#define _POLY_H
#include <vector>
#include "match.h"
#include "combine.h"

class polynomial;  //forward declaration - class declaration later in file

class polyElement {
    public:
    	virtual ~polyElement() {};
	virtual void putTo(ostream & os) const;
	virtual bool isExpansible() = 0;
	virtual vector <polyElement *>
	    getExpansion(matchTrans <matchClass> &Mn,
	    	vector <polyElement *> elems) 
	    	{vector <polyElement *> pv(0); return pv; }
	int getFactor() const {return mult;}
	void addFactor(int f) {mult+= f;}
	void setFactor(int f) {mult= f;}
	int getNoSites() const {return noSites;}
	int getProb() const {return prob;}
	virtual bool equals (polyElement *pe) {return false;}
	virtual double lhsEvaluate(vector <polynomial *> &plist, 
    	    licencePlates::combinePlates &obslist, 
    	    vector <int> &whichSites) const = 0;
	virtual double rhsEvaluate(vector <polynomial *> &plist, 
    	    licencePlates::combinePlates &obslist, 
	    vector <int> &whichSites) const = 0;
    protected:
    	int prob;   	    // Probability multiplier
    	int noSites;	    // Number of sites in match
	int mult;   	    // Multiplier
	bool polyAddTo (vector <polyElement *> elems, 
	    polyElement *match);
    private:
    	
  
    friend ostream & operator << (ostream &os, const polyElement &pe);
};

//  This match is a true match on censored data - observable
//  X(M(T),C(S))
class matchTrue: public polyElement {
    public:
    	matchTrue(int n, int p= 1, int fact= 1)  
	   {noSites= n; prob= p; mult= fact;}
	~matchTrue() {}
	bool isExpansible() {return false;}
	void putTo (ostream &os) const;
	bool equals (polyElement *pe);
	
	double lhsEvaluate(vector <polynomial *> &plist, 
    	    licencePlates::combinePlates &obslist, 
    	    vector <int> &whichSites) const;
	double rhsEvaluate(vector <polynomial *> &plist, 
    	    licencePlates::combinePlates &obslist, 
	    vector <int> &whichSites) const;
    private:
    
    friend ostream & operator << (ostream &os, const matchTrue &pe);
    
};

// This match is an exact match of a particular type
// It expands into real matches of all lower types
class exactMatch: public polyElement {
    public: 
    	exactMatch(int n, matchClass &match, int p= 1, int fact= 1) :
	    mc(match) {noSites= n; prob= p; mult= fact;}
	~exactMatch() {}
	bool isExpansible() {return true;}
	vector <polyElement *> getExpansion(matchTrans <matchClass> &Mn,
	    	vector <polyElement *> elems);
	void putTo (ostream &os) const;
	bool equals (polyElement *pe);
	matchClass &getMatchClass() {return mc;}
	double lhsEvaluate(vector <polynomial *> &plist, 
    	    licencePlates::combinePlates &obslist, 
    	    vector <int> &whichSites) const 
	    	{cout << "ERROR!  This should never be called!\n";return 0;}
	double rhsEvaluate(vector <polynomial *> &plist, 
    	    licencePlates::combinePlates &obslist, 
	    vector <int> &whichSites) const 
	    	{cout << "ERROR!  This should never be called!\n";return 0;}
    private:
    	matchClass mc;   
	
    
    friend ostream & operator << (ostream &os, const exactMatch &pe);	
};

class matchByParts: public polyElement {
    public:
    	matchByParts(int n, matchClass &match, int p= 1, 
	    int fact= 1) : mc(match) {noSites= n; prob= p; mult= fact;}
	~matchByParts() {}
	bool isExpansible() {return false;}
	void putTo (ostream &os) const;
	bool equals (polyElement *pe);
	matchClass &getMatchClass() {return mc;}
	double lhsEvaluate(vector <polynomial *> &plist, 
    	    licencePlates::combinePlates &obslist, 
    	    vector <int> &whichSites) const;
	double rhsEvaluate(vector <polynomial *> &plist, 
    	    licencePlates::combinePlates &obslist, 
	    vector <int> &whichSites) const;
    private:
    	
    	matchClass mc;  
    friend ostream & operator << (ostream &os, const matchByParts &pe);
     	
};


class polynomial {

    public:
    	polynomial() :elements(0), noSites(0), Mn(1){};
	polynomial(int n);
	~polynomial();
	void putTo (ostream &os) const;
	int getNoSites() const {return noSites;}
	int length() const {return elements.size();}
	int noTrans() const {return Mn.getNoClasses();}
	const polyElement *getElement(int i) const {return elements[i];}
    private:
    	vector <polyElement *> elements;
	int noSites;
	matchTrans <matchClass> Mn;
	
	polynomial (const polynomial &);
	const polynomial &operator= (const polynomial &);
	
    	void expandPoly();  	// Expand terms of polynomial
	void gatherPoly();  	// Gather like terms
    	void deleteElement(polyElement *e);
	void gatherElement (polyElement *e);
    friend ostream & operator << (ostream &os, const polynomial &p);
};



#endif
