#ifndef _READPLATES_H
#define _READPLATES_H
#include <iostream>
#include <string>
#include <vector>
#include <ctime>
//#include <strstream>
#include <sstream>
#include "hoursmins.h"

// Implementations of functions are found in readplates.cpp


namespace licencePlates {

class plateReadError {
    public:
    	plateReadError(const string &err, int line= noLineNo) :
	    errName(err), lineNo(line) {}
    	const string & getError() const {return errName;}
	const int getLineNo() const {return lineNo;}
	const bool validLineNo() const{return (lineNo != noLineNo);}
    private:
    	static const int noLineNo= -1;
    	string errName;
    	int lineNo;
    friend ostream& operator << (ostream &os, const plateReadError &err);
};

class plate {   // Class for a single licence plate observation
    public:
		//plate(): reg(0), myTime(0), comment(0) {}
        plate(string const &raw);  //  Construct a plate from a string
	const string &getReg() const {return reg;}
	const string &getComment() const {return comment;}
	const int getHrs() const{return myTime.getHrs();}
	const int getMins() const {return myTime.getMins();}
	const string getTimeStr() const {return myTime.getTimeStr();}
   private:
        string reg; // Registration part
        hoursMins::basicTime myTime;
        string comment;   // Extra notes on this plate (if any)
	
    friend ostream& operator << (ostream &os, const plate &thisPlate);
    friend bool operator == (const plate &left, const plate & right);
};

class plateList {   // List of information from licence plates
    public:
    	plateList ():
	    noVehicles(0) {}
        plateList (string const &fname);  // Construct a list from a file
	static const int maxLineLen= 1000;
	static const int lineTypeInfo= 1;
	static const int lineTypeComment= 2;
	static const int lineTypeData= 3;
	const string &getName() const {return name;}
	const int getNoVehicles() const {return noVehicles;}
	const int size() const {return list.size();}
	plate &getElement(int &i) {return list[i];}
	const plate &getElement(int i) const {return list[i];}
	void clearList() {list.clear();}
    private: // Data first
        vector <plate> list;	// All vehicle plates
        int noVehicles;     	// No. of vehicles (from #INFO line)
	string name;	    	// Name of list used
	    // Private functions
	void tidyup (); // Deal with a failed constructor
	static int parseInfoLine (string); // Parse a #INFO line
	static int typeOfLine (string);  // Parses a data line to determine type
    friend ostream & operator << (ostream &os, const plateList &plates);
};

} //End of licenceplate namespace

#endif
